/*==========================================================================
//
//      hal_arm_at91_icbnet.c
//
//      HAL misc board support code for ARM AT91SAM7X-based INTRA Controller
//      board Version 2.10 using optical LAN I/F KSZ8041.
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 2006 Andrew Lunn <andrew.lunn@ascom.ch>
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    andrew lunn
// Contributors: Andrew Lunn, John Eigelaar, oliver munz, robert brusa
// Date:         2009-02-06
// Purpose:      HAL board support
// Description:  Implementations of HAL board interfaces
//
//####DESCRIPTIONEND####
//
//========================================================================*/

#include <pkgconf/system.h>
#include <pkgconf/hal.h>
#include <cyg/hal/hal_diag.h>

#include <cyg/hal/hal_io.h>             // IO macros

void hal_plf_eth_init(void){

#if CYGHWR_HAL_ARM_KSZ8041_RESET_INIT

    cyg_uint32 stat;
    cyg_uint32 rmr;

    /* config 0..2 -> mode MII */
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB16, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB16, false);
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB4, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB4, false);
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB15, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB15, false);

    /* addr 0..2 -> 0x7 */
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB14, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB14, true);
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB13, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB13, true);
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB6, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB6, true);

    /* isolation -> no */
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB7, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB7, false);

    /* duplex -> yeah */
    HAL_ARM_AT91_GPIO_CFG_DIRECTION(AT91_GPIO_PB5, AT91_PIN_OUT);
    HAL_ARM_AT91_GPIO_PUT(AT91_GPIO_PB5, false);

    /* All the lines setup correctly. Now do a external reset and let the phy
      start up in the correct mode */
    HAL_READ_UINT32(AT91_RST+AT91_RST_RMR, rmr); /* Save the reset-mode */
    HAL_WRITE_UINT32(AT91_RST+AT91_RST_RMR, AT91_RST_RMR_KEY | (4 << 8)); /* reset-time in slow-clocks */
    HAL_WRITE_UINT32(AT91_RST+AT91_RST_RCR, AT91_RST_RCR_KEY | AT91_RST_RCR_EXTRST);

    do {
        HAL_READ_UINT32(AT91_RST+AT91_RST_RSR, stat);
    } while (!(stat & AT91_RST_RSR_NRST_SET));

    HAL_DELAY_US(200); /* KSZ8041 datasheet says >100us before MIIM-read */
    HAL_WRITE_UINT32(AT91_RST+AT91_RST_RMR, AT91_RST_RMR_KEY | (rmr & 0xffffff));
#endif
}

//--------------------------------------------------------------------------
// EOF at91icb_norm.c
